@@ -38,9 +38,13 @@ class Storage(BaseStorage):
 
     def put(self, path, bytes):
         path = unquote(path)
+        try:
+            key = md5(path).hexdigest() + '-' + path
+        except UnicodeEncodeError:
+            key = md5(path.encode('utf-8')).hexdigest() + '-' + path.encode('utf-8')
         
         r = [Mutation(column=self.data_fam + ':' + self.image_col, value=bytes)]
-        self.storage.mutateRow(self.table, md5(path).hexdigest() + '-' + path, r)
+        self.storage.mutateRow(self.table, key, r)
         return path
 
     def put_crypto(self, path):
@@ -51,22 +55,35 @@ class Storage(BaseStorage):
             raise RuntimeError("STORES_CRYPTO_KEY_FOR_EACH_IMAGE can't be True if no SECURITY_KEY specified")
 
         path = unquote(path)
+        try:
+            key = md5(path).hexdigest() + '-' + path
+        except UnicodeEncodeError:
+            key = md5(path.encode('utf-8')).hexdigest() + '-' + path.encode('utf-8')
 
         r = [Mutation(column=self.data_fam + ':' + self.crypto_col, value=self.context.config.SECURITY_KEY)]
-        self.storage.mutateRow(self.table, md5(path).hexdigest() + '-' + path, r)
-
+        self.storage.mutateRow(self.table, key, r)
+ 
     def put_detector_data(self, path, data):
         path = unquote(path)
+        try:
+            key = md5(path).hexdigest() + '-' + path
+        except UnicodeEncodeError:
+            key = md5(path.encode('utf-8')).hexdigest() + '-' + path.encode('utf-8')
 
         r = [Mutation(column=self.data_fam + ':' + self.detector_col, value=dumps(data))]
-        self.storage.mutateRow(self.table, md5(path).hexdigest() + '-' + path, r)
+        self.storage.mutateRow(self.table, key, r)
 
     def get_crypto(self, path):
         if not self.context.config.STORES_CRYPTO_KEY_FOR_EACH_IMAGE:
             return None
 
         path = unquote(path)
-        crypto = self.storage.get(self.table, md5(path).hexdigest() + '-' + path, self.data_fam + ':' + self.crypto_col)
+        try:
+            key = md5(path).hexdigest() + '-' + path
+        except UnicodeEncodeError:
+            key = md5(path.encode('utf-8')).hexdigest() + '-' + path.encode('utf-8')
+
+        crypto = self.storage.get(self.table, key, self.data_fam + ':' + self.crypto_col)
 
         if not crypto:
             return None
@@ -74,8 +91,12 @@ class Storage(BaseStorage):
 
     def get_detector_data(self, path):
         path = unquote(path)
+        try:
+            key = md5(path).hexdigest() + '-' + path
+        except UnicodeEncodeError:
+            key = md5(path.encode('utf-8')).hexdigest() + '-' + path.encode('utf-8')
 
-        data = self.storage.get(self.table, md5(path).hexdigest() + '-' + path, self.data_fam + ':' + self.detector_col)
+        data = self.storage.get(self.table, key, self.data_fam + ':' + self.detector_col)
 
         try:
             return loads(data[0].value)
@@ -84,8 +105,12 @@ class Storage(BaseStorage):
 
     def get(self, path):
         path = unquote(path)
+        try:
+            key = md5(path).hexdigest() + '-' + path
+        except UnicodeEncodeError:
+            key = md5(path.encode('utf-8')).hexdigest() + '-' + path.encode('utf-8')
 
-        r = self.storage.get(self.table, md5(path).hexdigest() + '-' + path, self.data_fam + ':' + self.image_col)
+        r = self.storage.get(self.table, key, self.data_fam + ':' + self.image_col)
         try:
             return r[0].value
         except IndexError: 
@@ -93,16 +118,24 @@ class Storage(BaseStorage):
 
     def exists(self, path):
         path = unquote(path)
+        try:
+            key = md5(path).hexdigest() + '-' + path
+        except UnicodeEncodeError:
+            key = md5(path.encode('utf-8')).hexdigest() + '-' + path.encode('utf-8')
 
-        r = self.storage.get(self.table, md5(path).hexdigest() + '-' + path, self.data_fam + ':' + self.image_col)
+        r = self.storage.get(self.table, key, self.data_fam + ':' + self.image_col)
 
         return len(r) != 0
 
     def remove(self,path):
         path = unquote(path)
+        try:
+            key = md5(path).hexdigest() + '-' + path
+        except UnicodeEncodeError:
+            key = md5(path.encode('utf-8')).hexdigest() + '-' + path.encode('utf-8')
 
         r = [Mutation(column=self.data_fam + ':' + self.image_col, isDelete=True)]
-        self.storage.mutateRow(self.table, md5(path).hexdigest() + '-' + path, r)
+        self.storage.mutateRow(self.table, key, r)
 
     def resolve_original_photo_path(self,filename):
         return filename
